/*******************************************************************************
 * Copyright (c) 2007 BEA Systems, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    wharley@bea.com - initial API and implementation
 *******************************************************************************/

package targets.filer;

import org.eclipse.jdt.apt.pluggable.tests.annotations.GenClass6;

/**
 * Processing this class should result in removal of the previously
 * generated type and creation of a new one, resulting in a compilation error.
 */
@GenClass6(name="XxxGenerated02", pkg="gen6")
public class Parent02 {
	gen6.Generated02 _gen;
}

