/*******************************************************************************
 * Copyright (c) 2015 Salesforce, Inc.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     igor@ifedorenko.com - initial API and implementation
 *******************************************************************************/
package targets.AnnotationProcessorTests.Bug471995;

public class B {
  public B(@Bug471995 missing.Type p1, @Bug471995 missing.Type p2) {}
}
